#!/usr/bin/env python
# -*- coding: utf-8 -*-

# =================================================================
#
# JMAHTMLParser: modified HTMLParser
#
# =================================================================

import urlparse
from HTMLParser import *

class JMAHTMLParser(HTMLParser):
    def __init__(self):
        HTMLParser.__init__(self)
    
    def goahead(self, end):
        rawdata = self.rawdata
        i = 0
        n = len(rawdata)
        while i < n:
            match = self.interesting.search(rawdata, i) # < or &
            if match:
                j = match.start()
            else:
                j = n
            ## only change 'i <= j' from 'i < j'
            if i <= j: self.handle_data(rawdata[i:j])
            i = self.updatepos(i, j)
            if i == n: break
            startswith = rawdata.startswith
            if startswith('<', i):
                if starttagopen.match(rawdata, i): # < + letter
                    k = self.parse_starttag(i)
                elif startswith("</", i):
                    k = self.parse_endtag(i)
                elif startswith("<!--", i):
                    k = self.parse_comment(i)
                elif startswith("<?", i):
                    k = self.parse_pi(i)
                elif startswith("<!", i):
                    k = self.parse_declaration(i)
                elif (i + 1) < n:
                    self.handle_data("<")
                    k = i + 1
                else:
                    break
                if k < 0:
                    if end:
                        self.error("EOF in middle of construct")
                    break
                i = self.updatepos(i, k)
            elif startswith("&#", i):
                match = charref.match(rawdata, i)
                if match:
                    name = match.group()[2:-1]
                    self.handle_charref(name)
                    k = match.end()
                    if not startswith(';', k-1):
                        k = k - 1
                    i = self.updatepos(i, k)
                    continue
                else:
                    if ";" in rawdata[i:]: #bail by consuming &#
                        self.handle_data(rawdata[0:2])
                        i = self.updatepos(i, 2)
                    break
            elif startswith('&', i):
                match = entityref.match(rawdata, i)
                if match:
                    name = match.group(1)
                    self.handle_entityref(name)
                    k = match.end()
                    if not startswith(';', k-1):
                        k = k - 1
                    i = self.updatepos(i, k)
                    continue
                match = incomplete.match(rawdata, i)
                if match:
                    # match.group() will contain at least 2 chars
                    if end and match.group() == rawdata[i:]:
                        self.error("EOF in middle of entity or char ref")
                    # incomplete
                    break
                elif (i + 1) < n:
                    # not the end of the buffer, and can't be confused
                    # with some other construct
                    self.handle_data("&")
                    i = self.updatepos(i, i + 1)
                else:
                    break
            else:
                assert 0, "interesting.search() lied"
        # end while
        if end and i < n:
            self.handle_data(rawdata[i:n])
            i = self.updatepos(i, n)
        self.rawdata = rawdata[i:]


class ExtractPrefecture(JMAHTMLParser):
    def __init__(self):
        JMAHTMLParser.__init__(self)
        self.prefecture = []
        self.is_in_map = False

    def handle_starttag(self, tag, attrs):
        if (tag == 'map') & (('name', 'point') in attrs):
            self.is_in_map = True

        elif (tag == 'area') & self.is_in_map:
            attrs = dict(attrs)
            attrs_qs = attrs['href'].split('?')[1]
            params = urlparse.parse_qs(attrs_qs)
            self.prefecture.append((attrs['alt'],
                                    params['prec_no'][0]))
    
    def handle_endtag(self, tag):
        if tag == 'map':
            self.is_in_map = False


class ExtractBlock(ExtractPrefecture):
    def __init__(self):
        JMAHTMLParser.__init__(self)
        self.block = []
        self.is_in_map = False
        self.amedas_or_sokkou = []

    def handle_starttag(self, tag, attrs):
        if (tag == 'map') & (('name', 'point') in attrs):
            self.is_in_map = True

        elif (tag == 'area') & self.is_in_map:
            attrs = dict(attrs)

            try:
                a_or_s = attrs['onmouseover'][21:-2].split(',')[0][1]
            except KeyError:
                a_or_s = ''
            self.amedas_or_sokkou.append(a_or_s)

            attrs_qs = attrs['href'].split('?')[1]
            params = urlparse.parse_qs(attrs_qs)
            if 'block_no' in params:
                self.block.append((attrs['alt'],
                                   str(params['block_no'][0])))


class ExtractTable(JMAHTMLParser):
    def __init__(self):
        JMAHTMLParser.__init__(self)
        self.table = []
        self.tr = []
        self.is_in_td = False
        self.is_in_tr = False
        self.include_img = False
        
    def handle_starttag(self, tag, attrs):
        if (tag == 'tr') & (('class', 'mtx') in attrs):
            self.is_in_tr = True
        elif (tag == 'td') & self.is_in_tr:
            self.is_in_td = True
        elif (tag == 'img') & self.is_in_td:
            self.include_img = True

    def handle_endtag(self, tag):
        if (tag == 'td') & self.is_in_td:
            self.is_in_td = False
            self.include_img = False
        elif (tag == 'tr') & self.is_in_tr:
            if self.tr != []:
                self.table.append(self.tr)
                self.tr = []
            self.is_in_tr = False
    
    def handle_data(self, data):
        if self.is_in_td:
            if data == '':
                if self.include_img:
                    pass
                else:
                    self.tr.append('--')
            else:
                self.tr.append(data)
