#!/usr/bin/env python
# -*- coding: utf-8 -*-

from datetime import date, timedelta
import csv
import jmalib

def ask_date():
    print("Beginning date of data you want.")
    d_start_year = int(raw_input("    year? "))
    d_start_month = int(raw_input("    month? "))
    d_start_date = int(raw_input("    date? "))
    d_start = date(d_start_year, d_start_month, d_start_date)

    print("Final date of data you want.")
    d_end_year = int(raw_input("    year? "))
    d_end_month = int(raw_input("    month? "))
    d_end_date = int(raw_input("    date? "))
    d_end = date(d_end_year, d_end_month, d_end_date)

    return (d_start, d_end)

def ask_location(jma):
    print("Choose location:")
    pref_list = jma.extract_prefecture()
    for i in range(len(pref_list)):
        p = pref_list[i]
        print("    %s: %s" % (i, p[0].decode('sjis').encode('utf-8')))
    
    pref_num = int(raw_input("  Choose prefecture (number): "))
    pref_params = {'prec_ch': pref_list[pref_num][0],
                   'prec_no': pref_list[pref_num][1]}

    extracted_blocks = jma.extract_block(pref_params)
    block_list = extracted_blocks[0]
    amedas_or_sokkou = extracted_blocks[1]
    for i in range(len(block_list)):
        b = block_list[i]
        print("    %s: %s" % (i, b[0].decode('sjis').encode('utf-8')))
    
    block_num = int(raw_input("  Choose block (number): "))
    block_params = {'block_ch': block_list[block_num][0],
                    'block_no': block_list[block_num][1]}

    pref_params.update(block_params)
    return (pref_params,
            amedas_or_sokkou[block_num])

def ask_header_sub():
    print('  Hourly or 10 min?')
    print('    1: hourly    2: 10 min.')
    which_table = int(raw_input('  Choose 1 or 2: '))
    return which_table

def ask_header(amedas_or_sokkou):
    hourly_s_table_header = ['年月日', '時間', '気圧 (hPa; 現地)',
                             '気圧 (hPa; 海面)', '降水量 (mm)', '気温 (oC)',
                             '露点温度 (oC)', '蒸気圧 (hPa)', '湿度 (%)',
                             '風速 (m/s)', '風向', '日照時間 (h)',
                             '全天日射 (MJ/m3)', '降雪 (cm)', '積雪 (cm)',
                             '天気', '雲量', '視程 (km)']
    ten_min_s_table_header = ['年月日', '時間', '気圧 (hPa; 現地)',
                              '気圧 (hPa; 海面)', '降水量 (mm)', '気温 (oC)',
                              '相対湿度 (%)', '平均風速 (m/s)', '平均風向',
                              '最大瞬間風速 (m/s)', '最大瞬間風向',
                              '日照時間 (h)']
    hourly_a_table_header = ['年月日', '時間', '降水量 (mm)', '気温 (oC)',
                             '風速 (m/s)', '風向', '日照時間 (h)', '降雪 (cm)',
                             '積雪 (cm)']
    ten_min_a_table_header = ['年月日', '時間', '降水量 (mm)', '気温 (oC)',
                              '平均風速 (m/s)', '平均風向',
                              '最大瞬間風速 (m/s)', '最大瞬間風向',
                              '日照時間 (h)']
    which_table = 0
    while True:
        which_table = ask_header_sub()
        if which_table == 1:
            if amedas_or_sokkou == 'a':
                return (which_table, hourly_a_table_header)
            elif amedas_or_sokkou == 's':
                return (which_table, hourly_s_table_header)
        elif which_table == 2:
            if amedas_or_sokkou == 'a':
                return (which_table, ten_min_a_table_header)
            elif amedas_or_sokkou == 's':
                return (which_table, ten_min_s_table_header)
        else:
            print('  ** Warning: choose 1 or 2!! **')
            continue

def main():
    jma = jmalib.JMA()
    
    asked_date = ask_date()
    d_start = asked_date[0]
    d_end = asked_date[1]
    d = d_start

    which_location = ask_location(jma)
    location_params = which_location[0]
    amedas_or_sokkou = which_location[1]

    which_header = ask_header(amedas_or_sokkou)
    table_header = which_header[1]

    stat_csv_file_name = raw_input("  saving file name? ")
    stat_csv = open(stat_csv_file_name, 'w')
    csv_writer = csv.writer(stat_csv)

    csv_writer.writerow([i.decode('utf-8').encode('sjis') \
                         for i in table_header])

    while d <= d_end:
        print("  Retrieve data of %s..." % d.isoformat())
        
        if which_header[0] == 1:
            jma_table = jma.hourly_table(d, location_params, amedas_or_sokkou)
        elif which_header[0] == 2:
            jma_table = jma.ten_min_table(d, location_params, amedas_or_sokkou)
            
        for row in jma_table:
            row_with_date = [d.isoformat()] + row
            csv_writer.writerow(row_with_date)
        d += timedelta(days=1)
    print("Done.")


if __name__ == '__main__':
    main()
    
