#!/usr/bin/env python
# -*- coding: utf-8 -*-

# =================================================================
#
# jmalib.py: get weather statistics from JMA
#
# =================================================================

import urllib
from JMAHTMLParser import ExtractTable, ExtractPrefecture, ExtractBlock

class JMA:
    def __init__(self):
        self.view_path = "http://www.data.jma.go.jp/obd/stats/etrn/view/"
        self.select_path = "http://www.data.jma.go.jp/obd/stats/etrn/select/"
        
    def hourly_table(self, date, location_params, amedas_or_sokkou):
        if amedas_or_sokkou == 'a':
            view_param = '"'
        else:
            view_param = ''
        
        table_params_dict = {
            'year': date.year,
            'month': date.month,
            'day': date.day,
            'elm': 'hourly',
            'view': view_param
            }
        table_params_dict.update(location_params)
        params = urllib.urlencode(table_params_dict)

        hourly_url = self.view_path + 'hourly_' + amedas_or_sokkou + '1.php'\
            + '?' + params
        f = urllib.urlopen(hourly_url)

        parser = ExtractTable()
        parser.feed(f.read())
        parser.close()
        return parser.table

    def ten_min_table(self, date, location_params, amedas_or_sokkou):
        if amedas_or_sokkou == 'a':
            view_param = '"'
        else:
            view_param = ''
        
        table_params_dict = {
            'year': date.year,
            'month': date.month,
            'day': date.day,
            'elm': 'minutes',
            'view': view_param
            }
        table_params_dict.update(location_params)
        params = urllib.urlencode(table_params_dict)

        ten_min_url = self.view_path + '10min_' + amedas_or_sokkou + '1.php'\
            + '?' + params
        f = urllib.urlopen(ten_min_url)

        parser = ExtractTable()
        parser.feed(f.read())
        parser.close()
        return parser.table

    def extract_prefecture(self):
        f = urllib.urlopen(self.select_path + 'prefecture00.php')

        parser = ExtractPrefecture()
        parser.feed(f.read())
        parser.close()
        return parser.prefecture

    def extract_block(self, pref_params_dict):
        params = urllib.urlencode(pref_params_dict)
        f = urllib.urlopen(self.select_path + 'prefecture.php' + '?' + params)

        parser = ExtractBlock()
        parser.feed(f.read())
        parser.close()
        return [parser.block, parser.amedas_or_sokkou]
